/** 
 *   Camera Control for STAR I.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <time.h>
#include <getopt.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <errno.h>

#include <curses.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "my_stdio.h"
#include "my_math.h"
#include "camera.h"
#include "fits_io.h"
#include "version.h"
#include "camera_tools.h"
#include "camera_xpa.h"

int cam;
int done = 0;

int readline_flag = 1;
int interactive_flag = 1;

/* The names of functions that actually do the manipulation. */
int com_help  __P((char *));
int com_quit  __P((char *));
int com_test  __P((char *));
int com_time  __P((char *));
int com_temperature  __P((char *));
int com_etime  __P((char *));
int com_exposure  __P((char *));
int com_palette  __P((char *));
int com_get  __P((char *));
int com_focus  __P((char *));
int com_shade  __P((char *));
int com_clear  __P((char *));
int com_gain  __P((char *));
int com_dark  __P((char *));
int com_bias  __P((char *));
int com_show_pos __P((char *));
int com_config __P((char *));
int com_display __P((char *));
int com_note __P((char *));

int com_temp_reg __P((char *));

int com_nroi __P((char *));
int com_roi_expose __P((char *));
int com_roi __P((char *));

int com_box_x0 __P((char *));
int com_box_y0 __P((char *));
int com_box_x_len __P((char *));
int com_box_y_len __P((char *));

int com_box_get __P((char *));
int com_box_set __P((char *));

int com_mouse_get __P((char *));
int com_chdir __P((char *));


/* A structure which contains information on the commands this program
   can understand. */

typedef struct {
	char *name;		/* User printable name of the function. */
	int show;               /* show or hide command */
	rl_icpfunc_t *func;	/* Function to call to do the job. */
	char *param;            /* Parameters for this function.  */
	char *doc;		/* Documentation for this function.  */
} COMMAND;

COMMAND commands[] = {
	{ "test", 0, com_test,"", "[!]" },
	{ "time", 1, com_time,"", "Set date & time on camera controller" },
	{ "temp", 1, com_temperature,"", "Get current CCD temperature" },
	{ "etime", 1, com_etime, "[time]", "Get/Set exposure time (sec)" },
	{ "treg", 1, com_temp_reg,"[on|off]", "Get/Set teperature regulator" },
	{ "expose", 1, com_exposure,"[etime]", "Exposure (F5)" },
	{ "dark", 1, com_dark,"", "Dark integration" },
	{ "bias", 1, com_bias,"", "Bias integration" },

	{ "get", 1, com_get,"{filename.fits}", 
	  "Get current image from the camera controller" },
	{ "display", 1, com_display,"{filename.fits}", 
	  "Display filename.fits image" },
	{ "note", 1, com_note,"{filename.fits note text}", 
	  "Put note in header of filename.fits" },
	{ "focus", 1, com_focus,"", "100x100 pixel focus integration" },
	{ "shade", 1, com_shade,"", "Test video display memory" },
	{ "clear", 1, com_clear,"", "Clear video display memory" },
	{ "gain", 1, com_gain,"[1|4]", "Get/Set system gain factor" },
	{ "showp", 1, com_show_pos,"[&]", 
	  "Show current position (HA, Dist. Pol.). &: run in xterm" },
	{ "conf", 1, com_config,"", "Edit current configuration file" },
	{ "pal", 1, com_palette,"[sat|hsat] [min] [max]", 
	  "Chage current video palette (none:reset, sat: saturation(F6), hsat: hight saturation)" },

	{ "nroi", 0, com_nroi,"", "Get/Set ROI number" },
	{ "roiexp", 0, com_roi_expose,"", "Expose ROI" },
	{ "roi", 0, com_roi, "[x0 y0 xlen ylen]", "Get/Set ROI" },

	{ "gpointer", 0, com_mouse_get, "", 
	  "Get current pointer location and pixel value"},
	{ "x0", 0, com_box_x0,"", "Get/Set X origin of box" },
	{ "y0", 0, com_box_y0,"", "Get/Set Y origin of box" },
	{ "xlen", 0, com_box_x_len,"", "Get/Set length X of box" },
	{ "ylen", 0, com_box_y_len,"", "Get/Set length Y of box" },

	{ "gbox", 0, com_box_get,"", "Get box" },
	{ "sbox", 0, com_box_set,"", "Set box" },
	
	{ "cd", 0, com_chdir, "[path]", "Change working directory" },	
	{ "help", 0, com_help, "[cmd]", "Display this text" },
	{ "?", 1, com_help, "[cmd]", "Synonym for `help'" },
	{ "quit", 0, com_quit, "", "Quit ..." },
	{ "q", 1, com_quit, "", "Quit :-)" },
	{ (char *)NULL, 0, (rl_icpfunc_t *)NULL, (char *) NULL, (char *)NULL }
};



pid_t my_exec(char *command)
{
	pid_t pid;
	
	if (command == 0)
		return 1;
	pid = fork();
	if (pid == -1)
		return -1;
	if (pid == 0) {
		char *argv[4];
		argv[0] = "sh";
		argv[1] = "-c";
		argv[2] = command;
		argv[3] = 0;
		if( execve("/bin/sh", argv, environ) == -1){
			perror("execve");
			exit(1); /* ojo: execve no retorna si tiene exito. */
		}
		exit(127);
	}
	return pid;
}


int my_system (char *command) {
           int pid, status;

           if (command == 0)
               return 1;
           pid = fork();
           if (pid == -1)
               return -1;
           if (pid == 0) {
               char *argv[4];
               argv[0] = "sh";
               argv[1] = "-c";
               argv[2] = command;
               argv[3] = 0;
               execve("/bin/sh", argv, environ);
               exit(127);
           }
           do {
               if (waitpid(pid, &status, 0) == -1) {
                   if (errno != EINTR)
                       return -1;
               } else
                   return status;
           } while(1);
       }


/* Generator function for command completion.  STATE lets us
   know whether to start from scratch; without any state
   (i.e. STATE == 0), then we start at the top of the list. */
char *command_generator (char *text, int state)
{ 
	static int list_index, len;
	char *name;

	/* If this is a new word to complete, initialize now.  This
	   includes saving the length of TEXT for efficiency, and
	   initializing the index variable to 0. */
	if (!state) {
		list_index = 0;
		len = strlen (text);
	}

	/* Return the next name which partially matches from the
	   command list. */
	while ((name = commands[list_index].name)) {
		list_index++;
		if (strncmp (name, text, len) == 0)
			return (strdup(name));
	}

	/* If no names matched, then return NULL. */
	return ((char *)NULL);
}


char **fileman_completion (const char *text, int start, int end)
{
	char **matches;

	matches = (char **)NULL;

	/* If this word is at the start of the line, then it is a
	   command to complete.  Otherwise it is the name of a file in
	   the current directory.  */
	if (start == 0)
	  matches = rl_completion_matches (text, 
					   (rl_compentry_func_t *) command_generator);
	
	return (matches);
}

int f1_command(int ignore, int ignore2)
{
	rl_ding();
	printf("ACA: 0x%x 0x%x 0x%x\n", ignore, ignore2, rl_read_key() );

	return 0;
}


/* Tell the GNU Readline library how to complete.  We want to try to
   complete on command names if this is the first word in the line, or
   on filenames if not. */
void initialize_readline (void)
{
	Keymap km;
	//char f5_macro[]={"\\C-p\\n"};
	char f5_macro[]={"expose\\n"};
	char f6_macro[]={"pal sat\\n"};

	/* Allow conditional parsing of the ~/.inputrc file. */
	rl_readline_name = "stari";
	
	/* Tell the completer that we want a crack first. */
	rl_attempted_completion_function = fileman_completion;

	/* Read keybindings and variable assignments from filename */
	/* int rl_read_init_file (const char *filename); */

	km = rl_get_keymap_by_name("emacs"); 
	rl_macro_bind("\e[15~", f5_macro, km);
	rl_macro_bind("\e[17~", f6_macro, km);
}


/* Look up NAME as the name of a command, and return a pointer to that
   command.  Return a NULL pointer if NAME isn't a command name. */
COMMAND *find_command (char *name)
{
	int i;

	for (i = 0; commands[i].name; i++)
		if (strcmp (name, commands[i].name) == 0)
			return (&commands[i]);

	return ((COMMAND *)NULL);
}


/* Execute a command line. */
int execute_line (char *line)
{
  int i;
	COMMAND *command;
	char *word;
	int pos_null = 0;
	/* Isolate the command word. */
	i = 0;
	while (line[i] && WHITESPACE(line[i]))	i++;
	word = line + i;

	while (line[i] && !WHITESPACE (line[i])) i++;

	if (line[i]){
		pos_null = i;
		line[i++] = '\0';
	}
	command = find_command (word);
	if (!command) {
		if(pos_null)
			line[pos_null] = ' ';
		my_system(line);
		//fprintf (stderr, "%s: No such command for stari.\n", word);
		return (-1);
	}

	/* Get argument to command, if any. */
	while (WHITESPACE (line[i])) i++;

	word = line + i;

	/* Call the function. */
	return ((*(command->func)) (word));
}


/* The user wishes to quit using this program.  Just set DONE
   non-zero. */
int com_quit (char *arg)
{
	done = 1;
	return (0);
}

/* Print out help for ARG, or for all of the commands if ARG is not
   present. */
int com_help (char *arg)
{
	int i, j;
	int printed = 0;
	int cmdlen;
	const int param_legend = 22;

	for (i = 0; commands[i].name; i++) {
		if ((!*arg && commands[i].show) || 
		    (strcmp (arg, commands[i].name) == 0) ) {
			cmdlen = strlen(commands[i].name) + 
				strlen(commands[i].param) + 1;
			printf ("%s %s", commands[i].name, 
				commands[i].param);
			if (cmdlen >= param_legend) {
				printf("\n");
				cmdlen = 0;
			}
			for(j = cmdlen; j < param_legend; j++)
				printf(" ");
			printf ("%s.\n", commands[i].doc);

			printed++;
		}
	}

	if (!printed) {
		printf ("No commands match `%s'.  Possibilties are:\n", arg);

		for (i = 0; commands[i].name; i++) {
			/* Print in six columns. */
			if (printed == 6) {
				printed = 0;
				printf ("\n");
			}
			
			printf ("%s\t", commands[i].name);
			printed++;
		}

		if (printed)
			printf ("\n");
	}
	return (0);
}


#define COMMAND_LINE_LEGEND 80


char *local_readline(char *prompt)
{
        char *line = NULL;
        fd_set rfds;
        struct timeval tv;
        int retval;
        char c;
        int l;

	write(0, prompt, strlen(prompt));
        line = (char *) malloc(COMMAND_LINE_LEGEND); 

        /* Loop reading and executing lines until the user quits. */
        l = 0;
        do {
                FD_ZERO(&rfds);
                FD_SET(0, &rfds);
                tv.tv_sec = 10;
                tv.tv_usec = 0;
                retval = select(50, &rfds, NULL, NULL, &tv);
                c = 0;
                if (retval) {
                        if (FD_ISSET(0, &rfds)) {
                                /* Win telnet clients send char by char ... */
                                if( read(0, &c, 1) != 1) {
                                        free(line);
                                        return NULL;
                                }
                                *(line+l) = c;
                                l++;
                        }
                }
        } while(c != '\n' && l < COMMAND_LINE_LEGEND && !done);
        if ( *(line+l-2) == 0xd ) *(line+l-2) = 0;
        else *(line+l-1) = 0;

        return line;
}


int command_main(void)
{
	char *line, *s;

	if (readline_flag)
		initialize_readline ();	/* Bind our completer. */

	/* Loop reading and executing lines until the user quits. */
	for ( ; done == 0; ) {
		if( readline_flag)
			line = readline ("stari> ");
		else 
                        line = local_readline ("stari> ");

		if (!line)
			break;
		s = stripwhite (line);

		if (*s)
		{
			add_history (s);
			execute_line (s);
		}
		
		free (line);
	}
	return (0);
}


int return_code(int ret)
{
	if(ret < 0) {
		if (ret == -2)	printf(" camera busy\n");
		else printf(" fail\n");
	}/*else
		printf(" ret: %d\n", ret);
	 */
	return ret;
}


int show_mini_setup(void)
{
	int gain;
	int etime;

	cc_get_gain(cam, &gain);
	if(gain == 0) gain = 1;
	else if(gain == 1) gain = 4;
	cc_get_exposure_time(cam, &etime);
	printf("Gain: %d, ETime: %5.1f\n", gain, (double) etime / 10.0);

	return 0;
}


int show_min_max_value(void)
{ 
	int min, max;
	int autow;

	cc_get_auto_win(cam, &autow);
	if( autow == 1 ) {
		cc_get_win_max(cam, &max);
		cc_get_win_min(cam, &min);
		printf("Palette Min: %4d, Max %4d\n", min, max);
	}
	return 0;
}


int clear_buff(int fd)
{
	fd_set rfds;
        int fd_max = fd + 1;
        int retval;
	struct timeval tv;
	char c;

	do {
		tv.tv_sec = 0;
		tv.tv_usec = 100000;
		FD_ZERO(&rfds);
		FD_SET(fd, &rfds);
		retval = select(fd_max, &rfds, NULL, NULL, &tv);
		if (retval) {
			if( FD_ISSET(fd, &rfds) ) {
				read(fd, &c, 1);
			}
		}
	}while(retval > 0);

	return 0;
}

int key_in_buff(int fd)
{
	fd_set rfds;
        int fd_max = fd + 1;
        int retval;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 100000;
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	retval = select(fd_max, &rfds, NULL, NULL, &tv);
	if (retval) {
		if( FD_ISSET(fd, &rfds) ) {
			return 1;
		}
	}

	return 0;
}



int show_remaining_time(int time)
{
	int time_sec;
	double t;
	char key;
	struct timeval exp, now;

	time_sec = (int) rint((double) time / 10.0);
	if (interactive_flag) {
		gettimeofday(&now, NULL);
		exp.tv_sec = now.tv_sec + time_sec;
		exp.tv_usec = now.tv_usec + 
			(time_t) (time - time_sec * 10) * 100000;
		while(timercmp(&exp, &now, >)) {
			//sleep(1);
			t = (double) ( (double) (exp.tv_sec - now.tv_sec) +
				       (double) (exp.tv_usec - now.tv_usec) / 
				       1000000.0);
			printf("remaining %5.1f sec. [ESC ENTER] to abort  \r",
			       t);
			fflush(stdout);
			if (key_in_buff(1)) {
				read(1, &key, 1);
				//printf("Key: 0x%x\n", key);
				if( key == 0x1b ) {
					printf("aborting...\n");
					fflush(stdout);
					return 1;
				}
			}
			gettimeofday(&now, NULL);
		}
		printf("                                                      \r");
		fflush(stdout);
	} else {
		sleep(time_sec);
	}

	return 0;
}




int com_time (char *arg)
{
	time_t now;

	time(&now);
	cc_set_date_time(cam, now);

	return 0;
}


int com_temperature (char *arg)
{ 
 	int t;
	int ret;

	ret = cc_get_temperature(cam, &t);
	if (ret == 0)
		printf(" Currente camera head temperature is %d\n", t);
	else return_code(ret);

	return 0;
}


int com_etime (char *arg)
{
	int etime;
	int ret;

        if (arg[0] == 0) {
		ret = cc_get_exposure_time(cam, &etime);
		if(ret == 0)
			printf(" current exposure time is %.1f\n", 
			       (double) etime / 10.0);
		else return_code(ret);
                return ret;
        }
	etime = (int) my_round(atof(arg) * 10.0);
	if ( etime > 0)
		cc_set_exposure_time(cam, etime);
	
	return 0;
}


int com_gain (char *arg)
{
	int gain;

        if (arg[0] == 0) {
		cc_get_gain(cam, &gain);
		if ( gain == GAIN_HIGH ) gain = 4;
		else gain = 1;
                printf(" current gain is %d\n", gain);
                return 0;
        }

	gain = atoi(arg);
	switch(gain){
	case 1:
		gain = GAIN_LOW;
		break;
	case 4:
		gain = GAIN_HIGH;
		break;
	default:
		printf(" gain: bad argument %s\n", arg);
		return -1;
	}

	return return_code( cc_set_gain(cam, gain) );
}


int com_temp_reg (char *arg)
{
	int reg;
	char sta[4];

        if (arg[0] == 0) {
		cc_get_temp_regulation(cam, &reg);
		if ( reg == TEMP_REG_ON ) sprintf(sta,"on");
		else sprintf(sta,"off");
                printf(" temperature regulator is %s\n", sta);
                return 0;
        }

	if( strncmp(arg, "on", 2) == 0)
		reg = TEMP_REG_ON;
	else if(strncmp(arg, "off", 3) == 0) 
		reg = TEMP_REG_OFF;
	else {
		printf(" tempreg: bad argument %s\n", arg);
		return -1;
	}
	cc_set_temp_regulation(cam, reg);

	return 0;
}



int com_exposure (char *arg)
{
	int etime;

	if (arg[0] != 0)
		com_etime(arg);
	cc_get_exposure_time(cam, &etime);
#ifdef DEMO_MODE
	etime = 50;
#endif
	cc_act_exposure(cam);
	if( show_remaining_time(etime) == 1)
		reset_cc(cam);
	else{
		show_mini_setup();
		show_min_max_value();
	}

	return 0;
}


int com_dark (char *arg)
{
	int etime;

	if (arg[0] != 0)
		com_etime(arg);
	cc_get_exposure_time(cam, &etime);
	cc_act_dark(cam);
	if( show_remaining_time(etime) == 1)
		reset_cc(cam);
	else{
		show_mini_setup();
		show_min_max_value();
	}

	return 0;
}


int com_bias (char *arg)
{
	cc_act_bias(cam);
	show_mini_setup();
	return 0;
}


int com_get (char *arg)
{
	char filename[_POSIX_PATH_MAX];
	time_t t;
	int good_image_flag = 0;
	struct stat dummy;
	char buff[8];

	if (arg[0] == 0) {
		time(&t);
		sprintf(filename,"im%10ld.fits", t); 
		
	}else{
		sprintf(filename,"%s", arg);
	}
	
	if (stat(filename, &dummy) == 0 && interactive_flag) {
		clear_buff(1);
		printf("File %s exist, overwrite (N/y)?: ", filename);
		fflush(stdout);
		read(1, &buff, 1);
		clear_buff(1);
		switch (*buff) {
		case 'Y':
		case 'y':
		case 'S':
		case 's':
			break;
		default:
			printf(" Abort\n");
			return -1;
			break;
		}
	}
	switch (ct_read_save_file(cam, filename)) {
	case CT_IM_OK:
		good_image_flag = 1;
		printf("ok.\n");
		break;
	case CT_IM_SETUP:
		printf(" Error reading image setup.\n");
		break;
	case CT_IM_DIM:
		printf(" Invalid dimension.\n");
		break;
	case CT_IM_READ:
		printf(" Error reading image.\n");
		break;
	case CT_IM_FITS_ERR:
		printf(" FITS Error.\n");
		break;
	default:
		printf(" Unknown Error.\n");
	}
	/* si llega hasta aca despliego... */
	if (good_image_flag) {
		caxpa_display(filename);
	}
	
	return 0;
}


int com_display (char *arg)
{
	return caxpa_display(arg);;
}


int com_note (char *arg)
{ 
	struct fit_tag *ft;
	int i;
	char *file;
	char *str;
	
	i = 0;
	while (arg[i] && WHITESPACE(arg[i])) i++;
	file = arg + i;
	while (arg[i] && !WHITESPACE (arg[i])) i++;
	if (arg[i]){
		arg[i++] = '\0';
	}
	while (arg[i] && WHITESPACE(arg[i])) i++;
	str = arg + i;
	if ( *str == 0) {
		printf(" Bad text.\n");
		return -1;
	}
	ft = (struct fit_tag *) malloc(sizeof(struct fit_tag));
	ft->next = NULL;
	fits_tag_push(ft, "NOTE", str, "");
	image_add_tag(file, ft);
	fits_tag_free(ft);

	return 0;
}


int com_focus (char *arg)
{
	/* FIXME: debe ejecutarlo continuamante hasta que se toque una
           tecla */
	cc_act_focus(cam);
	return 0;
}


int com_shade (char *arg)
{
	cc_act_shade(cam);
	return 0;
}


int com_clear (char *arg)
{
	cc_act_clear(cam);
	return 0;
}


int com_nroi (char *arg)
{
	int v;

        if (arg[0] == 0) {
		cc_get_roi_index(cam, &v);
                printf(" current number of ROIs is %d\n", v);
                return 0;
        }

	v = atoi(arg);
	cc_set_roi_index(cam, v);

	return 0;
}

int com_roi_expose (char *arg)
{
	int etime;

	if (arg[0] != 0)
		com_etime(arg);
	cc_get_exposure_time(cam, &etime);
	cc_act_roi_acquisition(cam);
	if( show_remaining_time(etime) == 1)
		reset_cc(cam);
	else{
		show_mini_setup();
		show_min_max_value();
	}
	
	return 0;
}


int com_roi (char *arg)
{
	int xy[4], i;
	char *argp;

        if (arg[0] == 0) {
		cc_get_box_x0(cam, &xy[0]);
		cc_get_box_y0(cam, &xy[1]);
		cc_get_box_x_len(cam, &xy[2]);
		cc_get_box_y_len(cam, &xy[3]);
                printf(" current ROI is %d %d %d %d\n", 
		       xy[0], xy[1], xy[2], xy[3]);
		
                return 0;
        }

	for(i = 0; i < 4 && arg != NULL; i++) {
		xy[i] = (int) strtod(arg, &argp);
		arg = argp;
	}
	if ( i != 4 ) {
		printf(" roi: bad argument %s\n", arg);
		return -1;
	}
	cc_set_box_x0(cam, xy[0]);
	cc_set_box_y0(cam, xy[1]);
	cc_set_box_x_len(cam, xy[2]);
	cc_set_box_y_len(cam, xy[3]);
	cc_act_set_box(cam);
	return 0;
}


int com_box_x0 (char *arg)
{
	int v;

        if (arg[0] == 0) {
		cc_get_box_x0(cam, &v);
                printf(" X origin is %d\n", v);
                return 0;
        }

	v = atoi(arg);
	cc_set_box_x0(cam, v);

	return 0;
}


int com_box_y0 (char *arg)
{
	int v;

        if (arg[0] == 0) {
		cc_get_box_y0(cam, &v);
                printf(" Y origin  is %d\n", v);
                return 0;
        }

	v = atoi(arg);
	cc_set_box_y0(cam, v);

	return 0;
}


int com_box_x_len (char *arg)
{
	int v;

        if (arg[0] == 0) {
		cc_get_box_x_len(cam, &v);
                printf(" Length of X is %d\n", v);
                return 0;
        }

	v = atoi(arg);
	cc_set_box_x_len(cam, v);

	return 0;
}


int com_box_y_len (char *arg)
{
	int v;

        if (arg[0] == 0) {
		cc_get_box_y_len(cam, &v);
                printf(" Length of Y is %d\n", v);
                return 0;
        }

	v = atoi(arg);
	cc_set_box_y_len(cam, v);

	return 0;
}


int com_box_get (char *arg)
{
  cc_act_get_box(cam);

  return 0;
}


int com_box_set (char *arg)
{
  cc_act_set_box(cam);

  return 0;
}


int com_mouse_get (char *arg)
{
	struct mouse_pointer mp;

	cc_act_rcv_mouse_xyz(cam, &mp);
	printf(" x = %3d, y = %3d, val = %4d\n", mp.x, mp.y, mp.val);

	return 0;
}


int com_chdir (char *arg)
{
	char *path;
	int ret;

	if (*arg == 0)
		path = getenv("HOME");
	else
		path = arg;
	ret = chdir(path);
	if( ret != 0)
		perror("cd");

	return ret;
}


int com_palette (char *arg)
{
	int  min, max;
	int  min_fix, max_fix;
	char lut[4096];
	int i;
	int sat_flag = 0;
	char *np = stripwhite(arg);
        char *endp;

	if (np[0] == 0) {
		cc_set_auto_win(cam, 1);
		cc_act_create_lut(cam);
	}else{
		//cc_set_auto_win(cam, 1);
		cc_get_win_min(cam, &min);
		cc_get_win_max(cam, &max);
		printf("Current palette: min = %d, max = %d\n", min, max);  
		min_fix = -1;
		max_fix = -1;
		if (strncmp(np, "sat", 3) == 0) {
			sat_flag = 1;
			np += 3;
		}else if(strncmp(np, "hsat", 4) == 0) {
			sat_flag = 2;
			np += 4;
		}
		if (strlen(np)) {
			min_fix = (int) strtol(np, &endp, 10 );
			if(*endp != 0) {
				np = endp;
				max_fix = (int) strtol(np, &endp, 10 );
			}else{
				max_fix = min_fix;
				min_fix = -1;
			}
		}
		switch(sat_flag) {
		case 0: 
			if (min_fix == -1) min_fix = min;
			if (max_fix == -1) max_fix = max;
			for(i=min_fix;i<max_fix-1;i++)
				lut[i] = (char) 
					rint((double)(i-min_fix) * 255.0/
					     (double)(max_fix - min_fix));
			for(i=1;i<min_fix;i++) lut[i] = 0;
			for(i=max_fix-1;i<4096;i++) lut[i] = 255;
			break;
		case 1: /* sat */
			if (min_fix == -1) min_fix = min;
			if (max_fix == -1) max_fix = max;
			for(i=1;i<min_fix;i++) lut[i] = 0;
			for(i=min_fix;i<max_fix-1;i++)
				lut[i] = (char) 
					rint((double)(i-min_fix) * 255.0/
					     (double)(max_fix - min_fix));
			for(i=max_fix-1;i<4096;i++) lut[i] = 0;
			break;
		case 2: /* hsat */
			/* FIXME: */
			printf("\n\n*** NOt implemented yet ***\n\n");
			break;
		}
		lut[0] = 255;
		lut[4095] = 0;
		cc_act_send_lut(cam, lut);
	}

	return 0;
}


#if 0
int com_palette_anda (char *arg)
{
	int  min, max;
	char lut[4096];
	int i;
	int pal;
	int hsat = 128;

	if (arg[0] == 0) {
		cc_set_auto_win(cam, 1);
		cc_act_create_lut(cam);
	}else{
		//cc_set_auto_win(cam, 1);
		cc_get_win_min(cam, &min);
		cc_get_win_max(cam, &max);
		printf("Current palette: min = %d, max = %d\n", min, max);  
		if (strncmp(arg, "sat", 3) == 0) {
			for(i=1;i<min;i++) lut[i] = 0;
			for(i=min;i<max-1;i++)
				lut[i] = (char) rint((double)(i-min)*255.0/
						     (double)(max-min));
			for(i=max-1;i<4096;i++) lut[i] = 255;
			lut[0] = 255;
			lut[4095] = 0;
			cc_act_send_lut(cam, lut);
		}else if (strncmp(arg, "hsat", 3) == 0) {
			if(max < hsat) hsat = max; 
			for(i=0;i<max-hsat;i++) lut[i] = hsat;
			for(i=0;i<hsat;i++) lut[max-1-i] = 255-i;
			for(i=max-1;i<4096;i++) lut[i] = 255;
			lut[0] = 255;
			lut[4095] = 0;
			cc_act_send_lut(cam, lut);
		}else{
			pal = atoi(arg);
			if ( pal > 0 && pal <4095) {
				if (min < pal) min = pal;
				if (max < pal) max = pal;
				for(i=min;i<max-1;i++)
					lut[i] = (char) rint((double)(i-min)*255.0/
							     (double)(max-min));
				for(i=1;i<pal;i++) lut[i] = 0;
				for(i=max-1;i<4096;i++) lut[i] = 255;
				lut[0] = 255;
				lut[4095] = 0;
				cc_act_send_lut(cam, lut);
			}
		}

	}

	return 0;
}
#endif

int com_test (char *arg)
{
//	int  min, max;
//	char lut[4096];
//	int i;
#if 0
	int b = atoi(arg);
	char str[20];
	cc_act_move_mouse(cam, b, b);
	
	get_int_from_cc(cam, "W", &b);
	printf("  Inten: %d\n", b);
	put_int_to_cc(cam,"V", 100);
	memset(str, 0, 20);
	
	if( b != 0) return 0;
	
	get_int_from_cc(cam, arg, &b);
	printf("  Result: %d\n", b);
#endif
	//system("vim stari.conf");
	//my_exec("vim");
	cc_set_auto_win(cam, 0);
	cc_set_win_min(cam, 2000);
	cc_set_win_max(cam, 3500);
	cc_act_create_lut(cam);


	return 0;
}


int com_show_pos (char *arg)
{
	char buff[160];
	double ra, dec;

	get_object_position(&ra, &dec);
	if ( *arg == '&') {
		sprintf(buff,"xterm -e %s -l %f -r %f -d %f&",
			TRACK_COMMAND, geo_loc.longitude, ra, dec);

	}else{
		sprintf(buff,"%s -l %f -r %f -d %f",
			TRACK_COMMAND, geo_loc.longitude, ra, dec);
	}
	return system(buff);
}


int com_config (char *arg)
{
	char buff[160];
	char *editor = getenv("EDITOR");
	
	if( editor == NULL) editor = DEF_EDITOR;
	sprintf(buff,"%s  %s", editor, local_config_file);

	return system(buff);
}


static void cartel(void)
{
	static const char *msg[] =
	{
	"",
	"Camera Control for STAR I",
	"Version: "VERSION,
	"http://www.iar.unlp.edu.ar/~fede/pub/star_i",
	"",
	"(c) Copyright 2004, 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,",
	"Instituto Argentino de Radio Astronomia (IAR).",
	"",
	"This program is free software; you can redistribute it and/or",
	"modify it under the terms of the GNU General Public License as",
	"published by the Free Software Foundation; either version 2 of",
	"the License, or (at your option) any later version.",
	"",     
	"The author does NOT admit liability nor provide warranty for any",
	"of this software. This material is provided \"AS-IS\" in the hope",
	"that it may be useful for others.",
	""
	};
	unsigned j;
	for (j = 0; j < sizeof(msg)/sizeof(msg[0]); j++)
		fprintf(stderr, "\t%s\n", msg[j]);
	
    return;
}



int main(int argc, char * const argv[])
{
	int op;

	verbose_flag = PERR;
	while  ((op = getopt(argc, argv, "vn")) != EOF)
		switch( op ){
		case 'n':
                        readline_flag = 0;
			interactive_flag = 0;
                        break;
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
		default:
			printf("Use %s -[v (incremental)][n]\n", argv[0]);
			exit(-1);
		}
	cartel();
	cam = open_camera();

	if( cam > 0 ) {
		set_location();
		caxpa_open();
		command_main();
		close_camera(cam);
		caxpa_close();
	}else{
		PVERB(PERR," stari: can not connect to cc\n");
	}

        return 0;
}

